//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "connection.exec.h"

#include "glob_defs.h"
#include <libvtql/vtql_utils.h>
#include "vtrun.h"
#include "server.h"
#include "db.h"
#include "station.h"
#include "error_class.h"
#include <sstream>

class BufferQueryResult:
 public thread::CallBack<SearchQueue::SingleResult>::CBFunct{
//public SearchQueue::ResultCallB::CBFunct{
 public:
  explicit BufferQueryResult(SearchQueue& s,VTQL::ShowMode mode):
   ID(s.cbMatch.Add(this)),bFin(false),sq(s),smMode(mode){};
  ~BufferQueryResult(){sq.cbMatch.Del(ID);}

  virtual void operator()(const Arg&);
  bool Process(Connection_base::Outp_C&); //finshed?

  const size_t ID;
 private:
  thread::Mutex m;
  bool bFin;
  typedef std::list<SearchQueue::SqPageTyp> PageSet;
  PageSet list;
  SearchQueue& sq;
  const VTQL::ShowMode smMode;
};

void BufferQueryResult::operator()(const SearchQueue::SingleResult& res){
 if(res.iRegID!=ID)return;
 m.Lock();
 if(res.bEnd)bFin=true;
 else list.push_back(res.page);
 m.UnLock();
}

bool BufferQueryResult::Process
 (Connection_base::Outp_C& Outp){
 SearchQueue::SqPageTyp pcopy;
 m.Lock();
 while(!Outp.Full() && !list.empty()){
  pcopy=list.front();
  m.UnLock();
  Outp.GetPage(pcopy.PD->pmd,smMode);
  Outp<<pcopy;
  m.Lock();
  list.pop_front();
 }
 bool b=(bFin && list.empty());
 m.UnLock();
 return b;
}

Connection_exec::Connection_exec(int iS,VTQL::UserMode um,Server& s):
 Connection_base(iS,um),server(s),Inp(Con,glob_defs::iMaxInpBytes),
 Outp(Con,glob_defs::iMaxOutpLines),bQuitting(false){

 Outp.Add(glob_defs::cpVersion);
 if(um==VTQL::UM_NOBODY){
  SetQuit();
  Outp.Add("no access at this socket");
 }else Outp.Add(glob_defs::cpGreeting);
}

Connection_exec::~Connection_exec(){server.Control(false,Con);}

bool Connection_exec::Add2Poll(SockWait& p)const{
 p.AddSocket(Con,(bQuitting?0:POLLIN)|(Outp.Empty()?0:POLLOUT));

 return (Outp.Full() || bQuitting || !Inp.Lines());
}

bool Connection_exec::Run(bool bForceQuery){
 if(!Flush(true,true))return false;

 if(!bQuitting && !Outp.Full())Exec(bForceQuery);

 if(!Flush(true,true))return false;
 return (!bQuitting || !Outp.Empty());
}

bool Connection_exec::Flush(bool bR,bool bW){
 while(bR || bW){
  if(!Inp.Free()||bQuitting)bR=false;
  if(Outp.Empty())bW=false;
  if(!Poll(bR,bW))return false;

  if(bW)Outp.Write();
  bW=false;
  if(bR)Inp.Read();
  bR=false;
 }
 return true;
}

void Connection_exec::Run_Queryres(){
 if(queryresp->Process(Outp)){
  queryresp=NULL;
  Outp.Add("SUCCESS");
 }
}

void Connection_exec::Exec(bool bForceQuery){
 if(queryresp){Run_Queryres();return;}
 try{
 int iQueryType=-1;
 VTQL::Parse vtql;
 std::pair<std::string,bool> s=Inp.GetLine();
 if(!s.second)return;
 vtql.NewString(s.first);
 iQueryType=vtql.GetCom();
 if(iQueryType<0){Outp.Add("VOID");return;}


 VT_Run& vt_run=(*server.vt_runp); //short cuts
 Database& vt_db=(*server.vt_dbp);

 bool bad=true;
 #define INC_QUERY
 #define INC_CONFIG
 #define EXEC_COM(n,u,c)\
 if(n==iQueryType && VTQL::UM_##u==(bForceQuery?VTQL::UM_QUERY:umLogin)){\
  {c};bad=false;\
 }
 #include "vtql.syntax.inc"
 #undef INC_CONFIG
 #undef INC_QUERY
 #undef EXEC_COM
 if(bad)throw Error::Syntax(METH_NAME,"bad command");
 if(queryresp){Run_Queryres();return;}
 else if(!bQuitting)Outp.Add("SUCCESS");
 }
 #define CATCH(t)\
 catch(Error::t err){\
  Outp.Add(err);\
  Outp.Add("ERROR");\
 }
 CATCH(Syntax)
 CATCH(Sock)
 CATCH(Exec)
 #undef CATCH
}

void Connection_exec::ShowFreq(){
 int iF(server.vt_runp->GetFreq());
 if(iF<0)Outp.Add("unknown");
 else  Outp<< iF;
}

void Connection_exec::GetPgL(const std::string& _sMatch,VTQL::ShowMode _sm){
 SearchQueue::SmartRP p(new Region);
 p->NewString(_sMatch);
 queryresp=new BufferQueryResult(server.vt_dbp->sqSearch,_sm);
 server.vt_dbp->GetPageL(queryresp->ID,p);
}

void Connection_exec::ShowCmds(){
 using VTQL::cmAllCmds;
 Outp.Add(glob_defs::cpHelp);
 for(int i=0;cmAllCmds[i][0].iArgs>=0;++i){
  std::stringstream s;
  for(int j=0;cmAllCmds[i][j].iArgs>=0;++j){
   s << cmAllCmds[i][j].sWord;
   if(cmAllCmds[i][j].iArgs>0)
    for(int k=0;k<cmAllCmds[i][j].iArgs;++k)s << " _";
   
   if(cmAllCmds[i][j+1].iArgs>=0) s << ' ';
  }
  Outp.Add(s.str());
 }
}

