//content       pattern search class
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef REGION_H
#define REGION_H

#include <list>
#include <libvtql/thread_util.h>
#include <libvtql/smartptr.h>
#include <libvtql/vtql_parse.h>
#include "vtpage.h"
#include <libvtql/sock.h>

class Region:public VTQL::Region_basic{
 public: 
  virtual ~Region(){}
  Region();
  virtual void NewString(std::string,const std::string& ="/|\\");

  typedef std::list<vt_tv::VTPage_ID> SinglePageSet;
 
  struct PageData_VT:PageData{
   virtual ~PageData_VT(){};
   PageData_VT(const u_int8_t* _cp):cp(_cp){};
   virtual void GetLine(size_t,char*)const;
   const u_int8_t* cp;
  };

  //macros
  bool MatchStation(size_t ID)const{return IsIn(ID,0);}
  const SinglePageSet& GetSinglePages()const{return spsFastFind;}
  bool MatchVTPgInfo(const vt_tv::VTPage_Info& r)const{
   return (IsIn(r.iFreq,1) && IsIn(r.iPage,2) && IsIn(r.iSPage,3));
  }
  bool MatchPage(const vt_tv::PageTyp& vp)const{
   return (MatchVTPgInfo(vp) && MatchTxt(PageData_VT(vp.PD->pmd)));
  }
 private:
  SinglePageSet spsFastFind;
};

class SearchQueue:protected thread::Thread{
 public:
  SearchQueue():Thread(){};
  SearchQueue(const SearchQueue&);//no impl
  virtual ~SearchQueue(){};

  struct SqPageTyp;
  struct SingleResult;
  typedef SmartPtr<Region> SmartRP;

  void Add2Queue(const SingleResult&,SmartRP);
  
  typedef thread::CallBack<SingleResult> ResultCallB;
  ResultCallB cbMatch;

  void ManageThread(bool);
  virtual void* Run();
 private:
  mutable thread::Mutex m;

  struct MultQuery;
  class MultiQueryMap:public CachedExternIDMap<MultQuery>{
   public:
    MultiQueryMap():CachedExternIDMap<MultQuery>(){};
    virtual ~MultiQueryMap(){};

    size_t Get_NxtID(size_t);
  } mmQuery;
  thread::Semaphore sQueue;
};

struct SearchQueue::SqPageTyp:vt_tv::PageTyp{
 SqPageTyp():vt_tv::PageTyp(),iStID(0){};
 SqPageTyp(const vt_tv::PageTyp& p,size_t i):vt_tv::PageTyp(p),iStID(i){};
 virtual ~SqPageTyp(){};

 bool operator <(const SqPageTyp& p)const{
  return ((iStID==p.iStID)?(vt_tv::PageTyp::operator <(p)):(iStID<p.iStID));
 }
 size_t iStID;

 friend void operator <<(sock::Output&,const SqPageTyp&);
};

struct SearchQueue::SingleResult{
 size_t iRegID;
 SqPageTyp page;
 bool bEnd;

 SingleResult(size_t i,bool b):iRegID(i),bEnd(b){};
 SingleResult(size_t i,const SqPageTyp& p):iRegID(i),page(p),bEnd(false){};
 SingleResult(const SingleResult& s):
  iRegID(s.iRegID),page(s.page),bEnd(s.bEnd){};
 SingleResult& operator =(const SingleResult& s){
  iRegID=s.iRegID;page=s.page;bEnd=s.bEnd;return *this;}

 bool operator <(const SingleResult& r)const{
  return ((iRegID==r.iRegID)?(page<r.page):(iRegID<r.iRegID));
 }

};

struct SearchQueue::MultQuery{
 SmartRP rp;
 std::list<SqPageTyp> pgl;

 MultQuery(SmartRP p):rp(p){};
};

#endif
