//content       provides the server
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef SERVER_H
#define SERVER_H

#include <string>
#include <map>
#include <libvtql/smartptr.h>
#include <libvtql/sock.h>
#include <libvtql/vtql_parse.h>
#include "wait.h"
#include <libvtql/vtql_utils.h>
#include "connection.h"

class Server;
class SocketInfo{
 public:
  SocketInfo(const SocketInfo&); //no impl

  SocketInfo(const std::string& s,VTQL::UserMode u,Server& ser):
   um(u),sFile(s),server(ser){};

  void Handle(int);

  void Listen(){sbSock.Listen(sFile);}
  void Close(){sbSock.Close();}

  int GetFD(){lcpCon.clear();return sbSock.GetFD();}
  void SetMod(VTQL::UserMode u){um=u;}
  bool IsOpen()const {return (sbSock.IsOpen() || !lcpCon.empty());}

 private:
  friend class Server;
  friend void operator <<(sock::Output&,const Server&);

  sock::Socket_basic sbSock;
  VTQL::UserMode um;
  std::string sFile;

  typedef CachedInternIDMap<Connection_base,int> ConMap;
  ConMap lcpCon;
  Server& server;
};

class Database;
class VT_Run;
class Server{
 public:
  Server():vt_runp(NULL),iLockSock(-1){}; 
  Server(const Server&); //no impl

  void Add(size_t,const std::string&);
  void Del(size_t);
  SocketInfo& operator [](size_t);
  void Control(bool bL,int iS)
   {if(bL)iLockSock=iS;else if(iS==iLockSock)iLockSock=-1;}

  void Handle() ; //run only once!!!
  typedef thread::CallBack<vt_tv::PageTyp> EventCallB;
  EventCallB cbEvent;
  VT_Run* vt_runp;
  Database* vt_dbp;
  WaitClass wc;

  friend void operator <<(sock::Output&,const Server&);
 private:
  typedef CachedExternIDMap<SocketInfo> SocketMap;
  SocketMap smSockets;
  int iLockSock;
 
  SockWait pAll;

  void CleanUp();
  bool FillUp();
};

#endif
