//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "db.h"
#include "station.h"
#include "vtrun.h"
#include <libvtql/sock.h>
#include <libvtql/vtql_utils.h>
#include "vtpage.h"
#include "region.h"
#include "server.h"
#include <sstream>

void operator <<(sock::Output& o,const SearchQueue::SqPageTyp& t){
 std::stringstream s;
 s << t.iStID << '.';
 if(t.iFreq>0)s << t.iFreq << '.';
 else s << "unknown." ;
 s << std::hex << t.iPage << '.';
 s << t.iSPage << '.';
 s << std::dec << t.tTime << '.' ;
 s << std::hex << int(t.iFlags) << '.';
 for(vt_tv::VTPage_Info::VTPage_FastLinks::const_iterator i=t.lpiLinks.begin();
  i!=t.lpiLinks.end();++i){
  if(i!=t.lpiLinks.begin())s << ',';
  s << std::hex << i->iPage;
  if(i->iSPage != vt_tv::ANY_SUB) s << '.' << i->iSPage ;
 }
 o.Add(s.str());
}

template<typename T>
void operator <<(sock::Output& o,const T& t){
 typename std::stringstream s;
 s << t ;
 o.Add(s.str());
}

template void operator <<(sock::Output& o,const int& t);
template void operator <<(sock::Output& o,const std::string& t);

void operator <<(sock::Output& o,const VT_Run& t){
 t.Lock();
 {
  std::stringstream s;
  s << "vbi\t" ;
  s << VTQL::EscapeString(t.sVbi) <<  '\t';
  if(t.vbip!=NULL)s << "open";
  else s << "closed";
  o.Add(s.str());
 }{
  std::stringstream s;
  s << "video\t" ;
  s << VTQL::EscapeString(t.sVideo) <<  '\t';
  if(t.iVideodev>=0)s << "open";
  else s << "closed";
  o.Add(s.str());
 }{
  std::stringstream s;
  s << "channel\t" ;
  s << VTQL::EscapeString(t.sVidChan) <<  '\t';
  if(t.iVideodev>=0)s << "open";
  else s << "closed";
  o.Add(s.str());
 }
 t.Unlock();
}



extern "C" {
 #include <libvtql/extern/frequencies.h>
}

namespace FreqTab{
void operator <<(sock::Output& o,const GetList& t){
 for(int i=0;i<t.iE;++i){
  std::stringstream s;
  if(t.iSt>=0)
   s << chanlists[t.iSt].name << "::" << chanlists[t.iSt].list[i].name
     << '\t' << chanlists[t.iSt].list[i].freq;
  else s << chanlists[i].name;
  o.Add(s.str());
 }
}
}

void operator <<(sock::Output& o,const Database& t){
 for(Database::StationMap::const_iterator i=t.smStations.begin();
   i!=t.smStations.end();++i)
  o<<*(i->second);
}

void operator <<(sock::Output& o,const StationTyp& t){
 std::stringstream s;
 s << VTQL::EscapeString(t.sName) << '\t';
 s << t.iID << '\t';
 s << VTQL::EscapeString(t.sMatch) << '\t';
 s << t.iTimeout << '\t';
 s << t.psPages.size() << '\t';
 if(t.tomPage.size()){
  StationTyp::TOMap::const_iterator i=t.tomPage.end();
  s << (*--i)->tTime;
 }else s << time_t(0);
 o.Add(s.str());
}

void operator <<(sock::Output& o,const Server& t){
 for(Server::SocketMap::const_iterator j=t.smSockets.begin();
  j!=t.smSockets.end();++j){
  std::stringstream s;
  s << j->first << '\t';
  s << VTQL::cppUserModes[j->second->um] << '\t';
  s << VTQL::EscapeString(j->second->sFile) << '\t';
  s << (j->second->IsOpen()?"open":"closed") << '\t';
  s << j->second->lcpCon.size();
  o.Add(s.str());
 }
}

