//content       station-specific part of the database
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef STATION_H
#define STATION_H

#include <set>
#include "vtpage.h"
#include "region.h"
#include <libvtql/sock.h>

class StationTyp{
 public:
  StationTyp(size_t);

  StationTyp(const StationTyp&); //not implemented

  bool Insert(const vt_tv::PageTyp&);
  void FillResultL(size_t,SearchQueue::SmartRP,SearchQueue&);
  void SetMatch(const std::string&);

  //macros
  std::string GetName()const{return sName;}
  void SetName(const std::string& s){sName=s;}
  void SetTO(int i){iTimeout=((i>0)?i:0);TO();}

  friend void operator <<(sock::Output&,const StationTyp&);
  
 private:
  size_t iID;
  std::string sName;

  std::string sMatch;
  SearchQueue::SmartRP regMatch;

  int iTimeout; //==0 : disabled
  time_t iCountPages;

  void TO();

  struct StPageTyp;
  typedef std::set<StPageTyp> PageSet;
  struct PageTyp_TO_less{
   bool operator ()(const PageSet::iterator&,const PageSet::iterator&)const;
  };

  typedef std::set<PageSet::iterator,PageTyp_TO_less> TOMap;

  PageSet psPages;
  TOMap tomPage;
}__attribute__((packed));

struct StationTyp::StPageTyp:vt_tv::PageTyp{
 StPageTyp():vt_tv::PageTyp(),tfakeT(0){};
 StPageTyp(const vt_tv::PageTyp& p,time_t t=0):vt_tv::PageTyp(p),tfakeT(t){};
 virtual ~StPageTyp(){};
 time_t tfakeT;
}__attribute__((packed));

inline bool StationTyp::PageTyp_TO_less::operator ()
 (const PageSet::iterator& i1,const PageSet::iterator& i2)const{
 return i1->tfakeT<i2->tfakeT;
}

#endif
