//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#include "vt_tv.h"

#include <sys/ioctl.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/videodev.h>
#include "vtpage.h"
#include "glob_defs.h"
#include "error_class.h"

extern "C" {
 #include "vbi.h"
 #include <libvtql/extern/vt.h>
}

//make sure we have only one instance
namespace vt_tv{VT_TV *vttvpInstance=NULL;};
using namespace vt_tv;
using std::string;

void vbi_send(vt_page* vtp){vttvpInstance->vbi_send(vtp);}

#define VBIP (static_cast<vbi_struct*>(vbip))

VT_TV::VT_TV():sVideo(""),sVbi(""),iVideodev(-1),vbip(NULL),iActF(-1),
 iIgnorePage(0),vtDatap(NULL){
 Lock();
 if(vttvpInstance){
  Unlock();
  throw Error::Program(METH_NAME,"2nd Instance of VT_TV");
 }
 vttvpInstance=this;
 Unlock();
}

VT_TV::~VT_TV(){
 CloseVideo();
 CloseVbi();
 Lock();vttvpInstance=NULL;Unlock();
}

bool VT_TV::SetFreq_U(int iF){
 if(iVideodev<0 || iF*16/1000<2)return false; //!!!
 if(iF!=iActF){ //something to do?
  unsigned long iFreq=((iF*16)+500)/1000;
  if(ioctl(iVideodev,VIDIOCSFREQ,&iFreq)<0){iActF=-1;return false;}
  iActF=(iFreq*1000+8)/16;
  iIgnorePage=glob_defs::iIgnorePages;
 }
 return true;
}

bool VT_TV::OpenVideo(){
 if(Video())return true;
 Lock();
 if(!sVideo.length()){Unlock();return false;}

 iVideodev=open(sVideo.c_str(),O_RDWR);
 if(iVideodev<0){Unlock();return false;}

 video_capability vc;
 if(ioctl(iVideodev,VIDIOCGCAP,&vc)<0){Unlock();return false;}

 video_channel vch;
 int i;
 for(i=0;i<vc.channels;++i){
  vch.channel=i;
  if(ioctl(iVideodev,VIDIOCGCHAN,&vch)<0){Unlock();return false;}
  if(
   (sVidChan=="" &&(vch.flags & VIDEO_VC_TUNER) && (vch.type & VIDEO_TYPE_TV))
   ||(sVidChan==vch.name)){
   if(ioctl(iVideodev,VIDIOCSCHAN,&vch)<0){iActF=-1;Unlock();return false;}
   break;
  }
 }
 if(i==vc.channels){Unlock();return false;}

 //get freq fron tuner
 unsigned iFreq;
 if(ioctl(iVideodev, VIDIOCGFREQ, &iFreq)<0)iActF=-1;
 else iActF=(iFreq*1000+8)/16;
 Unlock();

 return true;
}

void VT_TV::CloseVideo(){
 if(!Video())return;
 Lock();
 close(iVideodev);
 iVideodev=-1;
 iActF=-1;
 Unlock();
}

bool VT_TV::OpenVbi(){
 if(Vbi())return true;
 Lock();
 if(!sVbi.length()){Unlock();return false;}
 vbip=vbi_open(sVbi.c_str(),1,-1);
 if(!vbip){Unlock();return false;}

 iIgnorePage=glob_defs::iIgnorePages;
 Unlock();
 return true;
}

void VT_TV::CloseVbi(){
 if(!Vbi())return;
 Lock();
 vbi_close(VBIP);
 vbip=NULL;
 Unlock();
}

void VT_TV::SetVideo(const string& s){
 if(Video())
  throw Error::Exec(METH_NAME,"cannot change video device if open");
 Lock();sVideo=s;Unlock();
}

void VT_TV::SetVideoChan(const string& s){
 if(Video())
  throw Error::Exec(METH_NAME,"cannot change channel if video is open");
 Lock();sVidChan=s;Unlock();
}

void VT_TV::SetVbi(const string& s){
 if(Vbi())
  throw Error::Exec(METH_NAME,"cannot change vbi device if open");
 Lock();sVbi=s;Unlock();
}

void VT_TV::SetFreq(int iF){
 Lock();
 if(!SetFreq_U(iF)){
  Unlock();
  throw Error::Exec(METH_NAME,"error on setting frequency");
 }
 Unlock();
}

bool VT_TV::GetVT(PageTyp& vtD,int iT){
 vtD.iFreq=-1;
 Lock();
 if(!vbip || vtDatap){Unlock();return false;}//another thread is polling!
 vtDatap=&vtD;

 pollfd pPoll;
 pPoll.fd=VBIP->fd;
 pPoll.events=POLLIN;
 Unlock();
 if(poll(&pPoll,1,iT)>0){
  Lock();
  vbi_handler(VBIP,VBIP->fd);
 }else Lock();

 vtDatap=NULL;
 Unlock();
 return (vtD.iFreq>=0);
}

void VT_TV::vbi_send(void* vp){
#define VTP (*static_cast<vt_page*>(vp))
 if(iIgnorePage>0){
  --iIgnorePage;
  vtDatap->iFreq=-1;
 }else vtDatap->iFreq=((iActF>0)?iActF:0);
//if(VTP.errors)DGPR(VTP.errors);
//if(VTP.lines!=16777215)DGPR(VTP.lines);

 vtDatap->iPage=VTP.pgno;
 vtDatap->iSPage=VTP.subno;
 vtDatap->tTime=time(0);
 vtDatap->iFlags=VTP.flags;
 vtDatap->lpiLinks.clear();
 if(VTP.flof)
  for(int i=0;i<6;i++)
   vtDatap->lpiLinks.push_back(VTPage_ID(VTP.link[i].pgno,VTP.link[i].subno));

 for(size_t i=0;i<VT_HEIGHT;++i)
  memcpy(vtDatap->PD->pmd+i*VT_WIDTH,VTP.data[i],VT_WIDTH);
}
#undef VTP
#undef VBIP
