//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>
#include "command_exec.h"
#include "page_list.h"
#include <libvtql/error_base.h>
#include <readline/history.h>
#include <readline/readline.h>
#include <cstdlib>
#include <cstdio>
#include <signal.h>
#include <set>

using std::string;
using namespace VTQL;

static const char REDSTR[]=ESC_CHAR "[31m%s" ESC_CHAR "[0m";


static const char cpWarranty[]=
"Copyright (C) 2000-2002 Matthias\n"
" Kleinmann <Matthias.Kleinmann@gmx.de>\n"
"\n"
"This program is  free software;  you can\n"
"redistribute  it and/or modify it  under\n"
"the  terms  of  the GNU  General  Public\n"
"License   as   published  by   the  Free\n"
"Software Foundation; either version 2 of\n"
"the  License,  or (at  your option)  any\n"
"later version.\n"
"\n"
"This program  is distributed in the hope\n"
"that it will be useful,  but WITHOUT ANY\n"
"WARRANTY;   without  even  the   implied\n"
"warranty  of MERCHANTABILITY  or FITNESS\n"
"FOR  A PARTICULAR  PURPOSE.  See the GNU\n"
"General Public License for more details.\n"
"\n"
"You should  have received  a copy of the\n"
"GNU  General Public  License  along with\n"
"this program; if not, write to the\n"
"Free Software  Foundation,  Inc.,\n"
"59 Temple Place - Suite 330,\n"
"Boston, MA 02111-1307, USA.";

static const char cpVersion[]="console_vt version "CONSOLEVT_VERSION;

static const char cpGreeting[]=
"Copyright (C) 2000-2002 Matthias\n"
" Kleinmann <Matthias.Kleinmann@gmx.de>\n"
"VTQLserver  comes  with   ABSOLUTELY  NO\n"
"WARRANTY. This is free software, and you\n"
"are  welcome  to  redistribute it  under\n"
"certain conditions.\n"
"Type \"warranty\" for license details,\n"
"type \"help\" for help.";

static const char cpHelp[]=
    "*         available stations            \n"
    "*station  change station                \n"
    "          example: *ARD                 \n"
    "**        change to latest station      \n"
    "match     find pages within station     \n"
    "          examples: 100 ; 400.4 ; /stock\n"
    "!match    find pages globally           \n"
    "          examples: !..100 ; !..400.4 ; \n"
    "          !/stock                       \n"
    "+         next match                    \n"
    "-         previous match                \n"
    "l         latest match in list          \n"
    "list      show list of matches          \n"
    "1 ... 6   fastpage 1 to 6               \n"
    "<empty>   alias for '100,333'           \n"
    "umatch    updates within station        \n"
    "          examples: u100 ; u/stock      \n"
    "Umatch    global updates                \n"
    "          examples: U..100 ; U..400.4 ; \n"
    "?         toggle hidden text            \n"
    "help      this help\n"
    "warranty  licence\n"
    "quit,exit exit console_vt";

static bool bWaiting=false;
static void Handle_Term(int iSig){
 if(bWaiting)bWaiting=false;
 else {signal(iSig,SIG_DFL);raise(iSig);}
}

Exec::Exec(const std::string& s1,const std::string& s2):
 query(s1),event(s2),sOldCL(""){
 FindStation("*");
 using_history();
 signal(SIGHUP,SIG_IGN);
 signal(SIGPIPE,SIG_IGN);
 signal(SIGINT,Handle_Term);
}

string Exec::Prompt(){
 char* cp=readline((sSt+"> ").c_str());
 if(!cp)throw Error::Runtime(METH_NAME,"end of input");
 if(sOldCL!=cp && strlen(cp)>1){
  add_history(cp);
  sOldCL=cp;
 }
 string s=cp;
 std::free(cp);
 return s;
}

void Exec::ShowPage(PgInfo pid,bool bRev){
 char cp[500]="";
 std::sprintf(cp,"%u..%x.%x",unsigned(pid.iSt),
  unsigned(pid.iPage),unsigned(pid.iSPage));

 QM_PageAlevt qPage(query,cp,1);
 qPage.Wait(); 

 if(qPage.li.empty())std::puts("no match");
 else std::printf("%s",qPage.li.begin()->second.GetAnsi(bRev).c_str());
}

static size_t FindCL(const std::string& s1,const std::string& s2){
 size_t i;
 for(i=0;i<s1.size() && i<s2.size() && s1[i]==s2[i];++i);
 return i;
}

void Exec::ListStation(){
 QM_Stations qStations(query);
 qStations.Wait();

 time_t t=0;
 size_t sML=0;
 std::set<std::string> sAll;
 for(QM_Stations::iter i=qStations.sm.begin();i!=qStations.sm.end();++i){
  if(i->second.sName.size()>sML)sML=i->second.sName.size();
  if(t<i->second.tLatest)t=i->second.tLatest;
  sAll.insert(i->second.sName);
 }

 {
  char cpF[50];
  sprintf(cpF,ESC_CHAR "[0;0;4m%%-%is|%%4s|%%6s|*" ESC_CHAR "[0m\n",sML);
  std::printf(cpF,"Name"," ID ","#Pages");
 }
 std::set<std::string> sInc;
 for(QM_Stations::iter i=qStations.sm.begin();i!=qStations.sm.end();++i){
  size_t iMinL=0,iMinLnew=1;
  std::set<std::string>::iterator im;
  while(iMinLnew>iMinL){
   iMinL=iMinLnew;
   im=sInc.lower_bound(i->second.sName.substr(0,iMinL));
   if(im==sInc.end())break;
   iMinLnew=FindCL(*im,i->second.sName)+1;
   if(iMinLnew>i->second.sName.size())iMinLnew=i->second.sName.size();
  }
  while(iMinL<i->second.sName.size() &&
   (im=sAll.find(i->second.sName.substr(0,iMinL)))!=sAll.end()){
   iMinL=FindCL(*im,i->second.sName)+1;
   if(iMinL>i->second.sName.size())iMinL=i->second.sName.size();
  }

  sInc.insert(i->second.sName);

  char* cp=new char[iMinL+10];
  sprintf(cp,REDSTR,i->second.sName.substr(0,iMinL).c_str());
  char cpF[100];
  sprintf(cpF,"%%s%%-%is|%%4u|%%6u|%%c\n",sML-iMinL);

  std::printf(cpF,cp,i->second.sName.substr(iMinL).c_str(),
                unsigned(i->first),
                unsigned(i->second.iPageC),
                (t==i->second.tLatest)?'*':' ');
  delete[] cp;
 }
}

void Exec::FindStation(const string& sIn){
 QM_Stations qStations(query);
 qStations.Wait();
 if(qStations.sm.empty())
  throw Error::Runtime(METH_NAME,"no stations on server");

 if(sIn=="*"){
  time_t t=0;
  for(QM_Stations::iter i=qStations.sm.begin();i!=qStations.sm.end();++i)
   if(t<i->second.tLatest){
    t=i->second.tLatest;
    iSt=i->first;
    sSt=i->second.sName;
   }
 }else {
  size_t iM=0;
  for(QM_Stations::iter i=qStations.sm.begin();i!=qStations.sm.end();++i){
   size_t iP=FindCL(sIn,i->second.sName);
   if(iP>iM || (iP==sIn.size() && iP==i->second.sName.size())){
    iSt=i->first;
    sSt=i->second.sName;
    iM=iP;
   }
  }
 }
}

void Exec::Wait4Page(bool bAcSt,const std::string& sIn,bool bRev){
 char cp[50]="";
 std::sprintf(cp,"%u..",unsigned(iSt));
 EM_PageAlevt qPg(event,string(bAcSt?cp:"")+sIn);
 bWaiting=true;
 while(bWaiting){
  std::puts("update... (press CTRL-C to abort)");

  try{qPg.Wait();}catch(Error::Runtime){bWaiting=false;}

  if(bWaiting)std::printf("%s",qPg.alevtpg.GetAnsi(bRev).c_str());
 }
}

void Run(const std::string& sQSock,const std::string& sEvSock,bool bQuiet){
 Exec ex(sQSock,sEvSock);
 if(!bQuiet){
  std::puts(cpVersion);
  std::puts(cpGreeting);
  std::puts("");
 }

 PageList pgli(ex.query,ex.event,bWaiting);
 #define PgLi (PgInfo(pgli))
 #define Show(show) {if(!(PgLi.iFlags&PG_INVALID)){\
                pgli.ShowPos();\
                ex.ShowPage(PgLi,bRev);\
               }else if(show) std::puts("no match");\
              }
 std::string sIn;
 bool bRev=false;
 while(!std::feof(stdin)){
  try{sIn=ex.Prompt();}catch(Error::Runtime){std::puts("");return;}
  if(sIn.empty())sIn="100,333";

  bool bDone=false;
  switch(sIn[0]){ 
   case '1':case'2':case'3':case'4':case'5':case'6':
    if(sIn.size()!=1)break;
    if(PgLi.vFL.size()<size_t(sIn[0]-'0'))std::puts("no such fastlink");
    else{
     pgli.Search(ex.iSt,PgLi.vFL[sIn[0]-'1'].first,PgLi.vFL[sIn[0]-'1'].second);
     Show(true);
    }
    bDone=true;
    break; 
   case '*':
    if(sIn.size()==1){
     ex.ListStation();
     bDone=true;
    }else {
     ex.FindStation(sIn.substr(1));
     sIn="100,333";
    }
    break;
   case '!':
    pgli.Search(-1,sIn.substr(1));

    Show(false);
    bDone=true;
    break;
   case '?':
    if(sIn.size()!=1)break;

    bRev=!bRev;
    std::printf("reveal is now %s\n",bRev?"on":"off");
    Show(false);
    bDone=true;
    break;
   case '+':case '-': case 'l':
    if(sIn.size()!=1)break;

    switch(sIn[0]){
     case '+':++pgli;break;
     case '-':--pgli;break;
     case 'l':pgli.Latest();break;
    }

    Show(true);
    bDone=true;
    break;
   case 'U':
    ex.Wait4Page(false,sIn.substr(1),bRev);
    bDone=true;
    break;
   case 'u':
    ex.Wait4Page(true,sIn.substr(1),bRev);
    bDone=true;
    break;
  }
  if(sIn=="list"){pgli.ShowList();bDone=true;}
  if(sIn=="help"){std::puts(cpHelp);bDone=true;}
  if(sIn=="warranty"){std::puts(cpWarranty);bDone=true;}
  if(sIn=="quit"|| sIn=="exit")return;
 

  if(!bDone){
   pgli.Search(ex.iSt,sIn);

   Show(false);
  }
 }
}
