//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>
#include <cstdio>
#include <sstream>
#include "page_list.h"

using namespace VTQL;
const size_t PG_INVALID=0xF000;
static const std::string
  REVLINE=ESC_CHAR "M                                        \n"ESC_CHAR"M";
static inline void puts(const std::string& s){std::puts(s.c_str());}

void PageList::ShowPos()const{
 if(!pglp){std::puts("no page");return;}
 std::printf("(%i/%u)\n",iPos+1,unsigned(pglp->sPg.size()));
}

void PageList::ShowList()const{
 if(!pglp){std::puts("no page");return;}
 QM_Stations st(query);
 st.Wait();
 
 std::list<PgInfo>::const_iterator iL;
 for(iL=pglp->sPg.begin();iL!=pglp->sPg.end();++iL)
  std::printf("%s: %x.%x\n",st.sm[iL->iSt].sName.c_str(),
   unsigned(iL->iPage),unsigned(iL->iSPage));
}

PageList::operator PgInfo()const{
 if(!pglp||iter==pglp->sPg.end()){
  PgInfo PgInvalid;
  PgInvalid.iFlags=PG_INVALID;
  return PgInvalid;
 }
 return *iter;
}

void PageList::operator ++(){
 if(!pglp)return;
 if(iter!=pglp->sPg.end()){++iter;++iPos;}
 if(iter==pglp->sPg.end()){iter=pglp->sPg.begin();iPos=0;}
}

void PageList::operator --(){
 if(!pglp||pglp->sPg.empty())return;
 if(iter==pglp->sPg.begin()){iter=--pglp->sPg.end();iPos=pglp->sPg.size()-1;}
 else {--iter;--iPos;}
}

void PageList::Latest(){
 if(!pglp||pglp->sPg.empty())return;
 if(iter==pglp->sPg.end()){iter=pglp->sPg.begin();iPos=0;}

 QM_PageL::iter iL;
 size_t i=0;
 for(iL=pglp->sPg.begin();iL!=pglp->sPg.end();++iL,++i)
  if(iter->tTime<iL->tTime){iter=iL;iPos=i;}
}

void PageList::Search(int iS,int iP,int iSP){
 std::stringstream s;
 if(iP>=0) s << std::hex << iP;
 s << ".";
 if(iSP!=ANY_SUB && iSP>=0)s << std::hex << iSP;
 Search(iS,s.str());
}

void PageList::Search(int iS,const std::string& sIn){
 std::stringstream s;
 if(iS>=0)s<<iS << "..";
 s << sIn;
 EM_PageAlevt qPg(event,s.str());
 qPg.Wait(false);

 std::puts("searching...");
 do{
  pglp=new QM_PageL(query,s.str());
  pglp->Wait();

  if(pglp->error_flag.Err!=QMError_flag::QME_SUCCESS){
   puts(REVLINE+"invalid query"); 
   pglp=NULL;
   return;
  }
  if(pglp->sPg.empty()){
   puts(REVLINE+"waiting... (press CTRL-C to abort)");
   bWaiting=true;
   try{qPg.Wait();}catch(Error::Runtime){bWaiting=false;}

   if(!bWaiting){std::puts("aborted");pglp=NULL;return;}
   puts(REVLINE+"retring db search...");

   bWaiting=false;
  }
 }while(pglp->sPg.empty());
 iter=pglp->sPg.begin();iPos=0;
}


