/*! \file thread.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Undocumented basic thread stuff
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef THREAD_H
#define THREAD_H

#include <utility>
#include <semaphore.h>
#include <pthread.h>

/*!\brief Some thread stuff */
namespace thread{

class Semaphore{
 public:
  Semaphore(const Semaphore&); //no impl
  Semaphore(unsigned int i=0){sem_init(&s,0,i);}
//no need to:  ~Semaphore(){sem_destroy(&s);}

  void Wait(){sem_wait(&s);}
  bool TryW(){return !sem_trywait(&s);} //true on success
  void ForceZero(){while(TryW());}
  bool Post(){return !sem_post(&s);} //true on success
  int GetV()const{int i;sem_getvalue(&s,&i);return i;}

  Semaphore& operator ++(){Post();return *this;}
  Semaphore& operator --(){TryW();return *this;}
  operator int()const{return GetV();}
 private:
  mutable sem_t s;
};

class Mutex{
 public:
  Mutex(const Mutex&); //no impl
  Mutex(){pthread_mutex_init(&m,NULL);}
//no need to:  ~Mutex(){pthread_mutex_destroy(&m);}

  void Lock(){pthread_mutex_lock(&m);}
  bool TryL(){return !pthread_mutex_trylock(&m);} //true on success
  void UnLock(){pthread_mutex_unlock(&m);}
 private:
  pthread_mutex_t m;
};

class Thread{
 public:
  Thread(const Thread&); //no impl
  Thread(){};
  virtual ~Thread(){Stop();}
  
  template<class T>
  bool Start(); //success?
  std::pair<bool,void*> Stop();
 protected:
  virtual void* Run()=0;
  bool ExitRun(){return semExit.TryW();}
 private:
  pthread_t thrRun;
  Semaphore semExit;
};

};

#include "thread.tcc"
#endif
