//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>
#include "thread_util.h"

namespace thread{

template<typename T>
size_t CallBack<T>::Add(CBFunct* cbfp){
 m.Lock();
 int ID=++iID_C;
 cbmCBList.insert(std::make_pair(ID,cbfp));
 m.UnLock();
 return ID;
}

template<typename T>
bool CallBack<T>::Del(size_t ID){
 m.Lock();
 typename CBMap::iterator i=cbmCBList.find(ID);
 if(i==cbmCBList.end()){m.UnLock();return false;}
 cbmCBList.erase(i);
 m.UnLock();
 return true;
}

template<typename T>
void CallBack<T>::Run(const T& t){
 m.Lock();
 typename CBMap::iterator i=cbmCBList.begin();
 while(i!=cbmCBList.end()){
  std::pair<size_t,CBFunct*> iAct=*i;
  m.UnLock();
  (*iAct.second)(t);
  m.Lock();
  i=cbmCBList.upper_bound(iAct.first);
 }
 m.UnLock();
}

template<typename T>
FiFo<T>::FiFo(size_t s):tpBuf(new T[s]),iSize(s),iR(0),iW(0),sR(0),sW(s){}

template<typename T>
FiFo<T>::~FiFo(){delete[] tpBuf;}

template<typename T>
const T* FiFo<T>::Get_Start(bool bWait){
 if(bWait)sR.Wait();
 else if(!sR.TryW())return NULL;

 return tpBuf+iR;
}

template<typename T>
void FiFo<T>::Get_End(bool bGet){
 if(bGet){
  iR=(iR+1)%iSize;
  sW.Post();
 }else sR.Post();
}

template<typename T>
T* FiFo<T>::Put_Start(bool bWait){
 if(bWait)sW.Wait();
 else if(!sW.TryW())return NULL;

 return tpBuf+iW;
}

template<typename T>
void FiFo<T>::Put_End(bool bPut){
 if(bPut){
  iW=(iW+1)%iSize;
  sR.Post();
 }else sW.Post();
}

};
